# COMPLETE = 'done'
CONTROL_ARM = "control_arm"
DAYS = "days"
IN_PROGRESS_APPT = "in_progress"
PARTIAL = "partial"
FILLED = "filled"
PRINT_SELECTED = "print_selected"
SOLUTION = "SOLUTION"
CANCELLED = "cancelled"
DISPENSED = "dispensed"
# SUPPOSITORY = 'SUPPOSITORY'
# SYRUP = 'SYRUP'
# TABLET = 'TABLET'
# ORAL = 'oral'
# VIAL = 'vial'
# CAPSULE = 'capsule'
# IM = 'IM'
# IV = 'IV'
# MILLIGRAM = 'mg'
# MONTHS = 'months'
# WEEKS = 'weeks'
PRESCRIPTION_ACTION = "prescription"
